
JGTCLI_PROG_NAME="jgtcli"
JGTCLI_PROG_DESCRIPTION="JGTCLI Command Line Interface"
JGTCLI_EPILOG="Basically deals with Indicators data and the signal generation"



JGTADS_PROG_NAME="jgtads"
JGTADS_PROG_DESCRIPTION="Analysis Data Service Command Line Interface"
JGTADS_PROG_EPILOG="View Data and generated visualizations.Plot the chart for a given instrument and timeframe."


JGTAPY_PROG_NAME="jgtapycli"
JGTADS_PROG_EPILOG="Indicator data IDS generation, no signal.  It is meant to be a previous step to signal generation done in CDS and is not yet used in the current version of the system. (even though it is implemented, works and saves JGTPY_DATA/ids/... files)"
JGTADS_PROG_DESCRIPTION="JGTAPY Command Line Interface"



JGTMKS_PROG_NAME="jgtmkscli"
JGTMKS_PROG_DESCRIPTION="Market Data Service Command Line Interface"
JGTMKS_PROG_EPILOG="Market Data Service Command Line Interface"

